;  2 - 
; :
;      P3.2 (INT0),     P3.3 (INT1)

; :
;      (   MCS-51)      (       Timer0)
;       ( 12   ,  11,0592   )
;   .    (  ,    )    ,
;     -    ,        ,     
;      ,       
;       ,       (2-3  )

; :
;     - 256us
;      - 384us
;       - 320us (  64us   )

;   2   - 512us
;   2  - 768us
;   2     - 640us (  128us   )

;     - 768us
;     - 1024us
;       - 900us

;  :
;         -    
;      
;  TH0 = 0FFh       
;  (   )    
; 1.   INT0 -    Timer0 (TH0).  TH0 != 0FFh - .
;  Timer0     (TL0 = 0,TH0 = 0Fh), ,  
; 2.             (  ).   INT1,     (    -   INT1)
;  Timer0  -  (   > 1024us)
;    ( ,  ),       -    Timer0 (TH0).  TH0 != 0FFh -  (  < 768us)
;  Timer0     (TH0 = 0Fh, TL0 = 0), ,  
;  Timer0  -  (  > 1023us)
;   INT0 -    Timer0 (TH0).  TH0 != 0FFh - .  Timer0     (TH0 = 0Fh, TL0 = 0), ,  
;  Timer0     (TH0 = 0Fh, TL0 = 0), ,  

;    .1     OFF   ON (0->1),   .0  1->0

;  :
; 1.     (   - low level).
; 2. 1- -   (n>=3 && n<=127). 7-      0 (       127 ),   hi level    low level 
; 3. 2-n  - 
; 4. n+1  - XOR  ,   
;==============================================================================================================================================
RECEIVE_BUFFER_SIZE	set	32

RECEIVE_PIN		bit     P3.2 ; & P3.3 from invertor
LED_PORT		set     P2
LED_PIN     	bit     P2.0

TH0_PREAMB		set		0FCh
TH0_DATA		set		0FDh

NUM_PREAMB		set		4		; must be parity (>=4)
DATA_BITS		set		8

; receiver_state
WAIT_PREAMB		set		0
WAIT_DATA		set		1

;==============================================================================================================================================
;main_data	segment data
;rseg		main_data
;dseg    	at      2Fh

dseg    	at      8h
bit_count:			ds	1
len_count:			ds	1
receive_buff:		ds	RECEIVE_BUFFER_SIZE
receive_ptr:		ds	1
;receive_state:		ds	1

org      	2Fh
;main_idata	segment data
;rseg		main_idata
Stack:

;main_bdata	segment bit
;rseg		main_bdata
bseg            at      0
prev_change_data_front:	dbit    1	; previous level change (0 - is not a data front, 1 - was a data front)
preamb_ok:				dbit    1
receive_compl:			dbit    1

;==============================================================================================================================================
;main_code	segment code
;rseg    	main_code

cseg	at	0
	jmp		Init
org	03h
;	reti
	jmp		Int_0
org	0Bh
;	reti
	jmp		Timer0_Int
org	13h
;	reti
	jmp		Int_1

;==============================================================================================================================================
Int_0:
Int_1:
	push	PSW
	push    ACC
	mov		A,		TH0
	jnb		preamb_ok,Check_Preamb
;-----------------------------------------------------------
Check_Data:
	cjne	A,		#TH0_DATA+1,Check_for_max
	jbc		prev_change_data_front,Loop_receive_data		; this is a short signal, so if the previous level change was a data front, this level change is not a data front
	setb	prev_change_data_front
Check_for_max:
	jc		Error_Heandler
	mov		C,		RECEIVE_PIN								; get signal level
	push	0h												; save R0
	mov		R0,		receive_ptr								; get actual receive_ptr
	mov		A,		@R0
	rlc		A												; save signal level
	mov		@R0,	A
	pop		0h												; restore R0
	djnz	bit_count,Loop_receive_bits
	mov		A,		receive_ptr								; get actual receive_ptr
	clr		C
	subb	A,		#receive_buff							; get the amount of received data
	cjne	A,		receive_buff,Loop_receive_bytes			; compare with the given length (the first byte of the packet received)
	mov		len_count,receive_buff							; whole package accepted, get len_count
	dec		len_count
	clr		A												; clr ACC before first XOR
	push	0h												; save R0
	mov		R0,		#receive_buff							; set pointer to first byte
Loop_check_pack:	
	xrl		A,		@R0
	inc		R0
	djnz	len_count,Loop_check_pack
	clr		C
	subb	A,		@R0										; compare XOR
	pop		0h												; restore R0
	jnz		Error_Heandler
	setb	receive_compl
	clr		preamb_ok
	clr		TR0
	clr		EX0
	clr		EX1
Loop_receive_bytes:
	inc		receive_ptr										; update receive_ptr
Loop_receive_bits_2:
	mov		bit_count,#DATA_BITS							; restore bit_count for every receiving byte 
Loop_receive_bits:
	mov		TL0,	#0
	mov		TH0,	#TH0_DATA
End_Int_0_Int_1:
	pop     ACC
	pop		PSW
	reti

;-----------------------------------------------------------
Check_Preamb:
	cjne	A,		#0FFh,		Error_Heandler
	djnz	bit_count,Loop_receive_preamb
;	mov		receive_state,#WAIT_DATA
	setb	preamb_ok
	jmp		Loop_receive_bits_2
Loop_receive_preamb:
	mov		TL0,	#0
	mov		TH0,	#TH0_PREAMB
	jmp		End_Int_0_Int_1
;-----------------------------------------------------------
Error_Heandler:
	call	Set_default_state
	jmp		End_Int_0_Int_1
;==============================================================================================================================================
Timer0_Int:
	call	Set_default_state
	reti
;==============================================================================================================================================
Set_default_state:
;	mov		receive_state,#WAIT_PREAMB
	mov		receive_ptr,#receive_buff
	mov		bit_count,#NUM_PREAMB
	clr		prev_change_data_front
	clr		preamb_ok
	mov		TL0,	#0
	mov		TH0,	#TH0_PREAMB
	ret
;==============================================================================================================================================
Init:
	mov		SP,		#Stack
;== setup timers & interrupts for AT89C1051 ===================================================================================================
	mov		TMOD,	#0001b			; imer0 mode
	;		 		 ||||
	;		 		 ||++---------- imer0 mode (01 - 16-bit, 10 - 8-bit+reload, 11 - no valid, equ 01)
	;		 		 |+------------ /0: 0 - timer, 1 - counter
	;		 		 +------------- Gate_0: 0 - TR0 control, 1 - TR0 & INT0 control 

Start_receive:
	call	Set_default_state
	mov		TH0,	#0FFh			; write valid value to TH0 for first interrupt
	jb		RECEIVE_PIN,$			; wait low level, for first call INT0 interrupt

	mov		TCON,	#010101b		; imer0 control
	;		 		 ||||||
	;		 		 |||||+-------- IT0 - INT0 type: 0 - by level, 1 - by front
	;		 		 ||||+--------- IE0 - request INT0 (if IT0=1, autoclear in heandler)
	;		 		 |||+---------- IT1 - INT1 type: 0 - by level, 1 - by front
	;		 		 ||+----------- IE1 - request INT1 (if IT1=1, autoclear in heandler)
	;		 		 |+------------ TR0 - imer0 start 
	;		 		 +------------- TF0 - Timer0 overflow flag

	mov		IE,		#10000111b		; Interrupts
	;				 ||||||||
	;				 |||||||+------	EX0 - INT0 interrupt enable
	;				 ||||||+-------	0 - Timer0 interrupt enable
	;				 |||||+-------- EX1 - INT1 interrupt enable
	;				 ||||+--------- 1 - reserv, no use
	;				 |||+---------- ES  - reserv, no use
	;				 ||+----------- ET2 - reserv, no use
	;				 |+------------ reserv, no use
	;				 +-------------  - global interrupts enable
;==============================================================================================================================================
Wait_receive:
	jbc		receive_compl,To_do
	jmp     Wait_receive
To_do:
	mov		LED_PORT,	receive_buff+1
	jmp     Start_receive
;==============================================================================================================================================
Check_Receiver:
	mov		DPTR,	#Table_Receiver_State
	mov		A,		receive_state					; get actual receive_state
	rl		A										; *2, addr in word (2 byte)
	movc	A,		@A+DPTR							; read addr_hi
	push	ACC										; save addr_hi
	mov		A,		receive_state					; get actual receive_state
	rl		A										; *2, addr in word (2 byte)
	inc		A										; inc for addr_low
	movc	A,		@A+DPTR    						; read addr_low
	mov		DPL,	A								; load addr_low to DPTR
	pop		ACC										; restore addr_hi
	mov		DPH,	A								; load addr_hi to DPTR
	clr		A 										; clear ACC before jmp
	jmp		@A+DPTR									; jmp to current receive_state heandler
;==============================================================================================================================================
Wait_Preamb:
	jnb     preamb_ok,End_Check_Receiver

	jmp     End_Check_Receiver
;----------------------------------------------------------------------------------------------------------------------------------------------
Wait_Data:

	jmp     End_Check_Receiver
;==============================================================================================================================================
Table_Receiver_State:
	dw		#Wait_Preamb
	dw      #Wait_Data
;==============================================================================================================================================
end

;  2 -      
; 0    256us  512us
; 1    512us  768us
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
; 
